# -*- coding: utf-8 -*-

from django.db import models, migrations
from django.conf import settings
import phonenumber_field.modelfields


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Caller',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('blacklisted', models.BooleanField(default=False)),
                ('phone_number', phonenumber_field.modelfields.PhoneNumberField(unique=True, max_length=128)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Credential',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=30)),
                ('account_sid', models.CharField(max_length=34)),
                ('auth_token', models.CharField(max_length=32)),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL, on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.contrib.auth.models import User
from django.conf import settings

from twilio.rest import Client
from django_dynamic_fixture import G

from django_twilio.client import twilio_client
from django_twilio.models import Credential
from django_twilio.utils import discover_twilio_credentials


class TwilioClientTestCase(TestCase):

    def test_twilio_client_exists(self):
        self.assertIsInstance(twilio_client, Client)

    def test_twilio_client_sets_credentials(self):
        self.assertEqual(
            twilio_client.auth,
            (settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN)
        )

    def test_twilio_client_with_credentials_model(self):
        self.user = G(User, username='test', password='pass')
        self.credentials = G(
            Credential,
            name='Test Credentials',
            account_sid='AAA',
            auth_token='BBB',
            user=self.user,
        )

        credentials = discover_twilio_credentials(user=self.user)

        self.assertEqual(credentials[0], self.credentials.account_sid)
        self.assertEqual(credentials[1], self.credentials.auth_token)



# -*- coding: utf-8 -*-

from twilio.twiml.messaging_response import Message

from django_twilio.request import decompose

"""
Useful utility functions.
"""

import os

from django.http import HttpResponse
from django.conf import settings

from twilio.twiml.voice_response import VoiceResponse

from .models import Caller, Credential


def discover_twilio_credentials(user=None):
    """ Due to the multiple ways of providing SID / AUTH tokens through
        this package, this function will search in the various places that
        credentials might be stored.

        The order this is done in is:

        1. If a User is passed: the keys linked to the
           user model from the Credentials model in the database.
        2. Environment variables
        3. django.conf settings

        We recommend using environment variables were possible; it is the
        most secure option
    """

    SID = 'TWILIO_ACCOUNT_SID'
    AUTH = 'TWILIO_AUTH_TOKEN'

    if user:
        credentials = Credential.objects.filter(user=user.id)
        if credentials.exists():
            credentials = credentials[0]
            return credentials.account_sid, credentials.auth_token

    if SID in os.environ and AUTH in os.environ:
        return os.environ[SID], os.environ[AUTH]

    if hasattr(settings, SID) and hasattr(settings, AUTH):
        return settings.TWILIO_ACCOUNT_SID, settings.TWILIO_AUTH_TOKEN

    raise AttributeError(
        "Could not find {sid} or {auth} in environment variables, "
        "User credentials, or django project settings.".format(
            sid=SID,
            auth=AUTH,
        )
    )


def get_blacklisted_response(request):
    """Analyze the incoming Twilio request to determine whether or not to
    reject services. We'll only reject services if the user requesting service
    is on our blacklist.

    :param obj request: The Django HttpRequest object to analyze.
    :rtype: HttpResponse.
    :returns: HttpResponse if the user requesting services is blacklisted, None
        otherwise.
    """
    try:
        # get the `From` data from the request's payload.
        # Only supporting GET and POST.
        data = request.GET if request.method == 'GET' else request.POST
        frm = data['From']
        caller = Caller.objects.get(phone_number=frm)
        if caller.blacklisted:
            twilio_request = decompose(request)
            if twilio_request.type == 'voice':
                r = VoiceResponse()
                r.reject()
            else:
                # SMS does not allow to selectively reject SMS.
                # So, we respond with nothing, and twilio does not forward
                # the message back to the sender.
                r = Message()
            return HttpResponse(str(r), content_type='application/xml')
    except Exception:
        pass

    return None


def create_sub_account(user, twilio_client, friendly_name=None):
    new_account = twilio_client.api.accounts.create(friendly_name=friendly_name or user)
    new_credential = Credential.objects.create(user=user, account_sid=new_account.sid, auth_token=new_account.auth_token, name=new_account.friendly_name)
    return new_credential, new_account


def close_sub_account(account_sid, twilio_client):
    return twilio_client.accounts(account_sid).update(status="closed")


def close_sub_accounts_for_user(user, twilio_client):
    accounts = []
    for credential_obj in Credential.objects.filter(user=user):
        accounts.append(twilio_client.accounts(credential_obj.account_sid).update(status="closed"))
    return accounts


# Backwards compatibility for a poorly named function
discover_twilio_creds = discover_twilio_credentials



# -*- coding: utf-8 -*-

"""
This module provides useful django admin hooks that allow you to manage
various components through the django admin panel (if enabled).
"""

from django.contrib import admin

from .models import Caller, Credential


class CallerAdmin(admin.ModelAdmin):
    """This class provides admin panel integration for our
    :class:`django_twilio.models.Caller` model.
    """
    list_display = ('__str__', 'blacklisted')


admin.site.register(Caller, CallerAdmin)
admin.site.register(Credential)



# -*- coding: utf-8 -*-

"""
Twilio REST client helpers.
"""

from twilio.rest import Client

from .settings import TWILIO_ACCOUNT_SID, TWILIO_AUTH_TOKEN


twilio_client = Client(
    TWILIO_ACCOUNT_SID,
    TWILIO_AUTH_TOKEN,
)



# -*- coding: utf-8 -*-

"""
Useful decorators.
"""

from functools import wraps

from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from django.http import (
    HttpRequest, HttpResponse, HttpResponseForbidden, HttpResponseNotAllowed)

from twilio.twiml import TwiML as Verb
from twilio.request_validator import RequestValidator

from .settings import TWILIO_AUTH_TOKEN
from .utils import get_blacklisted_response


def twilio_view(f):
    """
    This decorator provides several helpful shortcuts for writing Twilio views.

        - It ensures that only requests from Twilio are passed through. This
          helps protect you from forged requests.

        - It ensures your view is exempt from CSRF checks via Django's
          @csrf_exempt decorator. This is necessary for any view that accepts
          POST requests from outside the local domain (eg: Twilio's servers).

        - It enforces the blacklist. If you've got any ``Caller``s who are
          blacklisted, any requests from them will be rejected.

        - It allows your view to (optionally) return TwiML to pass back to
          Twilio's servers instead of building an ``HttpResponse`` object
          manually.

        - It allows your view to (optionally) return any ``twilio.Verb`` object
          instead of building a ``HttpResponse`` object manually.

          .. note::
            The forgery protection checks ONLY happen if ``settings.DEBUG =
            False`` (aka, your site is in production).

    Usage::

        from twilio import twiml

        @twilio_view
        def my_view(request):
            r = twiml.Response()
            r.message('Thanks for the SMS message!')
            return r
    """
    @csrf_exempt
    @wraps(f)
    def decorator(request_or_self, *args, **kwargs):
        # When using `method_decorator` on class methods,
        # I haven't been able to get any class views.
        # i would like more research before just taking the check out.
        class_based_view = not isinstance(request_or_self, HttpRequest)
        if not class_based_view:
            request = request_or_self
        else:
            assert len(args) >= 1
            request = args[0]

        # Turn off Twilio authentication when explicitly requested, or
        # in debug mode. Otherwise things do not work properly. For
        # more information, see the docs.
        use_forgery_protection = getattr(
            settings,
            'DJANGO_TWILIO_FORGERY_PROTECTION',
            not settings.DEBUG,
        )
        if use_forgery_protection:

            if request.method not in ['GET', 'POST']:
                return HttpResponseNotAllowed(request.method)

            # Forgery check
            try:
                validator = RequestValidator(TWILIO_AUTH_TOKEN)
                url = request.build_absolute_uri()
                signature = request.META['HTTP_X_TWILIO_SIGNATURE']
            except (AttributeError, KeyError):
                return HttpResponseForbidden()

            if request.method == 'POST':
                if not validator.validate(url, request.POST, signature):
                    return HttpResponseForbidden()
            if request.method == 'GET':
                if not validator.validate(url, request.GET, signature):
                    return HttpResponseForbidden()

        # Blacklist check, by default is true
        check_blacklist = getattr(
            settings,
            'DJANGO_TWILIO_BLACKLIST_CHECK',
            True
        )
        if check_blacklist:
            blacklisted_resp = get_blacklisted_response(request)
            if blacklisted_resp:
                return blacklisted_resp

        response = f(request_or_self, *args, **kwargs)

        if isinstance(response, (str, bytes)):
            return HttpResponse(response, content_type='application/xml')
        elif isinstance(response, Verb):
            return HttpResponse(str(response), content_type='application/xml')
        else:
            return response
    return decorator



# -*- coding: utf-8 -*-
from django.http import HttpRequest
from django.core.handlers.wsgi import WSGIRequest

from .exceptions import NotDjangoRequestException


class TwilioRequest(object):
    '''
    Primarily a collection of key/values from a Twilio HTTP request.
    Also has some additional attributes to support development with the
    Twilio API
    '''

    def __init__(self, parameters):
        self._build_params(parameters)

    def _build_params(self, parameters):
        '''
        Build out the Twilio key/values from the parameters into attributes
        on this class.
        '''
        for key, value in parameters.items():
            if key == 'From':
                setattr(self, 'from_', value)
            else:
                setattr(self, key.lower(), value)
        if getattr(self, 'callsid', False):
            self.type = 'voice'
        elif getattr(self, 'messagesid', False):
            self.type = 'message'
        else:
            self.type = 'unknown'


def decompose(request):
    '''
    Decompose takes a Django HttpRequest object and tries to collect the
    Twilio-specific POST parameters and return them in a TwilioRequest object.
    '''
    if type(request) not in [HttpRequest, WSGIRequest]:
        raise NotDjangoRequestException(
            'The request parameter is not a Django HttpRequest object')
    if request.method == 'POST':
        return TwilioRequest(request.POST.dict())
    if request.method == 'GET':
        return TwilioRequest(request.GET.dict())



# -*- coding: utf-8 -*-

import sys


# Django settings for test_project project.

import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
from distutils.version import StrictVersion

import django

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


DEBUG = True

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Los_Angeles'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = ''

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = ''

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = ''

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = '/static/admin/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'j1wd@qqodn-r9h&o@0jj!uw^#pm5wcdu2^cdsax=hm+-mk705p'

# This is a temporary shim to allow the old style MIDDLEWARE_CLASSES to work
# We will forge a plan to remove at least the unsupported versions soon.
# Django 2.0 is the future, but 1.11 is still supported.
# This test_project though is simple enough that the restrictions are small.
if StrictVersion(django.__version__) < StrictVersion('2.0'):
    MIDDLEWARE_CLASSES = (
        'django.middleware.common.CommonMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    )
else:
    MIDDLEWARE = [
        'django.middleware.security.SecurityMiddleware',
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'django.middleware.clickjacking.XFrameOptionsMiddleware',
    ]

ROOT_URLCONF = 'test_project.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates')]
        ,
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'django.contrib.admindocs',


    # Use django-nose for running our tests:
    'django_nose',

    # django-twilio, of course!
    'django_twilio',
)

# Nose test settings.
TEST_RUNNER = 'django_nose.NoseTestSuiteRunner'

NOSE_ARGS = ['--with-coverage', '--cover-package=django_twilio']


# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

# django-twilio account credentials. These fields are required to use the REST
# API (initiate outbound calls and SMS messages).
TWILIO_ACCOUNT_SID = 'ACXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'
TWILIO_AUTH_TOKEN = 'YYYYYYYYYYYYYYYYYYYYYYYYYYYYYYYY'

# The default callerid will be used for all outgoing phone calls and SMS
# messages if not explicitly specified. This number must be previously
# validated with twilio in order to work. See
# https://www.twilio.com/user/account/phone-numbers#
TWILIO_DEFAULT_CALLERID = 'NNNNNNNNNN'



# -*- coding: utf-8 -*-

from django.db import models
from django.conf import settings

from phonenumber_field.modelfields import PhoneNumberField


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


class Caller(models.Model):
    """
    A caller is defined uniquely by their phone number.

    :param bool blacklisted: Designates whether the caller can use our
        services.
    :param char phone_number: Unique phone number in `E.164
        <http://en.wikipedia.org/wiki/E.164>`_ format.

    """
    blacklisted = models.BooleanField(default=False)
    phone_number = PhoneNumberField(unique=True)

    def __str__(self):
        return '{phone_number}{blacklist_status}'.format(
            phone_number=str(self.phone_number),
            blacklist_status=' (blacklisted)' if self.blacklisted else '',
        )

    class Meta:
        app_label = 'django_twilio'


class Credential(models.Model):
    """
    A Credential model is a set of SID / AUTH tokens for the Twilio.com API

        The Credential model can be used if a project uses more than one
        Twilio account, or provides Users with access to Twilio powered
        web apps that need their own custom credentials.

    :param char name: The name used to distinguish this credential
    :param char account_sid: The Twilio account_sid
    :param char auth_token: The Twilio auth_token
    :param key user: The user linked to this Credential

    """

    def __str__(self):
        return '{name} - {sid}'.format(name=self.name, sid=self.account_sid)

    name = models.CharField(max_length=30)

    user = models.OneToOneField(AUTH_USER_MODEL, on_delete=models.CASCADE)

    account_sid = models.CharField(max_length=34)

    auth_token = models.CharField(max_length=32)

    class Meta:
        app_label = 'django_twilio'



# -*- coding: utf-8 -*-

import os

from unittest import mock
from django.conf import settings
from django.http import HttpResponse
from django.test import Client, TestCase
from django.test.utils import override_settings
from django_dynamic_fixture import G
from twilio.twiml.messaging_response import Message
from twilio.twiml.voice_response import VoiceResponse

from django_twilio.models import Caller
from django_twilio.utils import discover_twilio_credentials
from .utils import TwilioRequestFactory
from .views import (response_view, str_view, bytes_view, verb_view,
                    BytesView, StrView, VerbView, ResponseView)


class TwilioViewTestCase(TestCase):

    def setUp(self):

        self.regular_caller = G(Caller, phone_number='+15005550000', blacklisted=False)
        self.blocked_caller = G(Caller, phone_number='+15005550001', blacklisted=True)

        self.factory = TwilioRequestFactory(
            token=settings.TWILIO_AUTH_TOKEN,
            enforce_csrf_checks=True,
        )

        # Test URIs.
        self.uris = []
        self.str_uri = '/test_app/decorators/str_view/'
        self.uris.append(self.str_uri)
        self.str_class_uri = '/test_app/decorators/str_class_view/'
        self.uris.append(self.str_class_uri)
        self.bytes_uri = '/test_app/decorators/bytes_view/'
        self.uris.append(self.bytes_uri)
        self.bytes_class_uri = '/test_app/decorators/bytes_class_view/'
        self.uris.append(self.bytes_class_uri)
        self.verb_uri = '/test_app/decorators/verb_view/'
        self.uris.append(self.verb_uri)
        self.verb_class_uri = '/test_app/decorators/verb_class_view/'
        self.uris.append(self.verb_class_uri)
        self.response_uri = '/test_app/decorators/response_view/'
        self.uris.append(self.response_uri)
        self.response_class_uri = '/test_app/decorators/response_class_view/'
        self.uris.append(self.response_class_uri)

    def _assertStatusCode(self, actual_code, expected_code, uri):
            return self.assertEqual(
                actual_code, expected_code,
                '%s != %s. Bad uri is: %s' % (actual_code, expected_code, uri)
            )

    def test_requires_get_or_post(self):
        c = Client(enforce_csrf_checks=True)
        with override_settings(DEBUG=False):
            for uri in self.uris:
                self._assertStatusCode(c.get(uri).status_code, 403, uri)
                self._assertStatusCode(c.post(uri).status_code, 403, uri)
                self._assertStatusCode(c.head(uri).status_code, 405, uri)
                self._assertStatusCode(c.options(uri).status_code, 405, uri)
                self._assertStatusCode(c.put(uri).status_code, 405, uri)
                self._assertStatusCode(c.delete(uri).status_code, 405, uri)

    def test_all_return_statuses_when_debug_true(self):
        c = Client(enforce_csrf_checks=True)
        with override_settings(DEBUG=True):
            for uri in self.uris:
                self._assertStatusCode(c.get(uri).status_code, 200, uri)
                self._assertStatusCode(c.post(uri).status_code, 200, uri)
                self._assertStatusCode(c.head(uri).status_code, 200, uri)
                self._assertStatusCode(c.options(uri).status_code, 200, uri)
                if uri.endswith('class_view/'):
                    self._assertStatusCode(c.put(uri).status_code, 405, uri)
                    self._assertStatusCode(c.delete(uri).status_code, 405, uri)
                else:
                    self._assertStatusCode(c.put(uri).status_code, 200, uri)
                    self._assertStatusCode(c.delete(uri).status_code, 200, uri)

    def test_allows_post(self):
        request = self.factory.post(self.str_uri)
        self.assertEqual(str_view(request).status_code, 200)

    def test_allows_get(self):
        request = self.factory.get(self.str_uri)
        self.assertEqual(str_view(request).status_code, 200)

    def test_class_view_allows_post(self):
        request = self.factory.post(self.str_class_uri)
        self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_decorator_preserves_metadata(self):
        self.assertEqual(str_view.__name__, 'str_view')

    def test_class_decorator_preserves_metadata(self):
        self.assertEqual(StrView.dispatch.__name__, 'dispatch')

    @override_settings(TWILIO_ACCOUNT_SID=None)
    @override_settings(TWILIO_AUTH_TOKEN=None)
    def test_missing_settings_return_forbidden(self):
        with override_settings(DEBUG=False):
            self.assertEqual(self.client.post(self.str_uri).status_code, 403)
            self.assertEqual(self.client.post(self.str_class_uri).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(self.client.post(self.str_uri).status_code, 200)
            self.assertEqual(self.client.post(self.str_class_uri).status_code, 200)

    def test_missing_signature_returns_forbidden(self):
        with override_settings(DEBUG=False):
            self.assertEqual(self.client.post(self.str_uri).status_code, 403)
            self.assertEqual(self.client.post(self.str_class_uri).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(self.client.post(self.str_uri).status_code, 200)
            self.assertEqual(self.client.post(self.str_class_uri).status_code, 200)

    def test_incorrect_signature_returns_forbidden(self):
        with override_settings(DEBUG=False):
            request = self.factory.post(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(str_view(request).status_code, 200)
        with override_settings(DEBUG=False):
            request = self.factory.get(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(str_view(request).status_code, 200)

    def test_incorrect_signature_returns_forbidden_class_view(self):
        with override_settings(DEBUG=False):
            request = self.factory.post(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(StrView.as_view()(request).status_code, 200)
        with override_settings(DEBUG=False):
            request = self.factory.get(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 403)
        with override_settings(DEBUG=True):
            self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_no_from_field(self):
        request = self.factory.post(self.str_uri)
        self.assertEqual(str_view(request).status_code, 200)

    def test_no_form_field_class_view(self):
        request = self.factory.post(self.str_class_uri)
        self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_from_field_no_caller(self):
        request = self.factory.post(self.str_uri, {'From': '+12222222222'})
        self.assertEqual(str_view(request).status_code, 200)

    def tst_form_field_no_caller_class_view(self):
        request = self.factory.post(self.str_class_uri, {'From': '+12222222222'})
        self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_blacklist_works(self):
        with override_settings(DEBUG=False):
            request = self.factory.post(self.str_uri, {'From': str(self.blocked_caller.phone_number)})
            response = str_view(request)
            r = Message()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )
        with override_settings(DEBUG=True):
            request = self.factory.post(self.str_uri, {'From': str(self.blocked_caller.phone_number)})
            response = str_view(request)
            r = Message()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )
        with override_settings(DEBUG=False):
            request = self.factory.post(self.verb_uri, {'From': str(self.blocked_caller.phone_number),
                                                        'callsid': 'some-call-sid', })
            response = verb_view(request)
            r = VoiceResponse()
            r.reject()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )
        with override_settings(DEBUG=True):
            request = self.factory.post(self.verb_uri, {'From': str(self.blocked_caller.phone_number),
                                                        'callsid': 'some-call-sid', })
            response = verb_view(request)
            r = VoiceResponse()
            r.reject()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )

    def test_black_list_works_class_view(self):
        with override_settings(DEBUG=False):
            request = self.factory.post(self.str_class_uri, {'From': str(self.blocked_caller.phone_number)})
            response = StrView.as_view()(request)
            r = Message()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )
        with override_settings(DEBUG=True):
            request = self.factory.post(self.str_class_uri, {'From': str(self.blocked_caller.phone_number)})
            response = StrView.as_view()(request)
            r = Message()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )

    def test_black_list_works_verb_class_view(self):
        with override_settings(DEBUG=False):
            request = self.factory.post(self.verb_class_uri, {'From': str(self.blocked_caller.phone_number),
                                                              'callsid': 'some-call-sid', })
            response = VerbView.as_view()(request)
            r = VoiceResponse()
            r.reject()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )
        with override_settings(DEBUG=True):
            request = self.factory.post(self.verb_class_uri, {'From': str(self.blocked_caller.phone_number),
                                                              'callsid': 'some-call-sid', })
            response = VerbView.as_view()(request)
            r = VoiceResponse()
            r.reject()
            self.assertEqual(
                response.content,
                str(r).encode('utf-8'),
            )

    def test_decorator_modifies_str(self):
        request = self.factory.post(self.str_uri)
        self.assertIsInstance(str_view(request), HttpResponse)

    def test_decorator_modifies_str_class_view(self):
        request = self.factory.post(self.str_class_uri)
        self.assertIsInstance(StrView.as_view()(request), HttpResponse)

    def test_decorator_modifies_bytes(self):
        request = self.factory.post(self.bytes_uri)
        self.assertIsInstance(bytes_view(request), HttpResponse)

    def test_decorator_modifies_bytes_class_view(self):
        request = self.factory.post(self.bytes_class_uri)
        self.assertIsInstance(BytesView.as_view()(request), HttpResponse)

    def test_decorator_modifies_verb(self):
        request = self.factory.post(self.verb_uri)
        self.assertIsInstance(verb_view(request), HttpResponse)

    def test_decorator_modifies_verb_class_view(self):
        request = self.factory.post(self.verb_class_uri)
        self.assertIsInstance(VerbView.as_view()(request), HttpResponse)

    def test_decorator_preserves_http_response(self):
        request = self.factory.post(self.response_uri)
        self.assertIsInstance(response_view(request), HttpResponse)

    def test_decorator_preserves_http_response_class_view(self):
        request = self.factory.post(self.response_class_uri)
        self.assertIsInstance(ResponseView.as_view()(request), HttpResponse)

    def test_override_forgery_protection_off_debug_off(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=False):
            request = self.factory.post(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 200)

    def test_override_forgery_protection_off_debug_off_class_view(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=False):
            request = self.factory.post(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_override_forgery_protection_off_debug_on(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=True):
            request = self.factory.post(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 200)

    def test_override_forgery_protection_off_debug_on_class_view(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=False, DEBUG=True):
            request = self.factory.post(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 200)

    def test_override_forgery_protection_on_debug_off(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=False):
            request = self.factory.post(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 403)

    def test_override_forgery_protection_on_debug_off_class_view(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=False):
            request = self.factory.post(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 403)

    def test_override_forgery_protection_on_debug_on(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=True):
            request = self.factory.post(
                self.str_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(str_view(request).status_code, 403)

    def test_override_forgery_protection_on_debug_on_class_view(self):
        with override_settings(DJANGO_TWILIO_FORGERY_PROTECTION=True, DEBUG=True):
            request = self.factory.post(
                self.str_class_uri,
                HTTP_X_TWILIO_SIGNATURE='fake_signature',
            )
            self.assertEqual(StrView.as_view()(request).status_code, 403)


class TwilioUtilTest(TestCase):
    def test_discover_twilio_credentials_environ(self):
        SID = 'TWILIO_ACCOUNT_SID'
        AUTH = 'TWILIO_AUTH_TOKEN'
        patched = {SID: u'environ-test-case-sid', AUTH: u'environ-test-case-auth', }
        with mock.patch.dict(os.environ, patched):
            sid, auth = discover_twilio_credentials()
        self.assertEqual(sid, patched[SID])
        self.assertEqual(auth, patched[AUTH])

    @override_settings()
    def test_discover_twilio_credentials_none(self):
        del settings.TWILIO_AUTH_TOKEN
        del settings.TWILIO_ACCOUNT_SID
        self.assertRaises(AttributeError, discover_twilio_credentials)



# -*- coding: utf-8 -*-

from django.conf import settings
from django.http import HttpResponse
from django.test import TestCase
from twilio.twiml.voice_response import VoiceResponse

from django_twilio.decorators import twilio_view
from django.views.generic import View
from django.utils.decorators import method_decorator

from django_twilio.views import (
    conference, dial, gather, play, record, say, sms, message
    )

from .utils import TwilioRequestFactory


@twilio_view
def response_view(request):
    """
    A simple test view that returns a HttpResponse object.
    """
    return HttpResponse(
        '<Response><Message>Hello from Django</Message></Response>',
        content_type='text/xml',
    )


class ResponseView(View):

    @method_decorator(twilio_view)
    def dispatch(self, request, *args, **kwargs):
        return super(ResponseView, self).dispatch(request, *args, **kwargs)

    def get(self, request):
        return HttpResponse(
            '<Response><Message>Hello from Django</Message></Response>',
            content_type='text/xml',
        )

    def post(self, request):
        return HttpResponse(
           '<Response><Message>Hello from Django</Message></Response>',
            content_type='text/xml',
        )


@twilio_view
def str_view(request):
    """
    A simple test view that returns a string.
    """
    return '<Response><Message>Hi!</Message></Response>'


class StrView(View):

    @method_decorator(twilio_view)
    def dispatch(self, request, *args, **kwargs):
        return super(StrView, self).dispatch(request, *args, **kwargs)

    def get(self, request):
        return '<Response><Message>Hi!</Message></Response>'

    def post(self, request):
        return '<Response><Message>Hi!</Message></Response>'


@twilio_view
def bytes_view(request):
    """
    A simple test view that returns ASCII bytes.
    """
    return b'<Response><Message>Hi!</Message></Response>'


class BytesView(View):

    @method_decorator(twilio_view)
    def dispatch(self, request, *args, **kwargs):
        return super(BytesView, self).dispatch(request, *args, **kwargs)

    def get(self, request):
        return b'<Response><Message>Hi!</Message></Response>'

    def post(self, request):
        return b'<Response><Message>Hi!</Message></Response>'


@twilio_view
def verb_view(request):
    """
    A simple test view that returns a ``twilio.Verb`` object.
    """
    r = VoiceResponse()
    r.reject()
    return r


class VerbView(View):

    @method_decorator(twilio_view)
    def dispatch(self, request, *args, **kwargs):
        return super(VerbView, self).dispatch(request, *args, **kwargs)

    def get(self, request):
        r = VoiceResponse()
        r.reject()
        return r

    def post(self, request):
        r = VoiceResponse()
        r.reject()
        return r


class SayTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.say_uri = '/test_app/views/say/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)
        self.request = self.factory.post(self.say_uri)

    def test_say_no_text(self):
        self.assertRaises(TypeError, say, self.request)

    def test_say_with_text(self):
        self.assertEqual(
            say(self.request, text='hi').status_code,
            200
        )


class PlayTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.play_uri = '/test_app/views/play/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_play_no_url(self):
        request = self.factory.post(self.play_uri)
        self.assertRaises(TypeError, play, request)

    def test_play_with_url(self):
        request = self.factory.post(self.play_uri)
        self.assertEqual(
            play(request, url='http://b.com/b.wav').status_code,
            200,
        )


class GatherTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.gather_uri = '/test_app/views/gather/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_gather(self):
        request = self.factory.post(self.gather_uri)
        self.assertEqual(
            gather(request).status_code,
            200,
        )


class RecordTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.record_uri = '/test_app/views/record/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_record(self):
        request = self.factory.post(self.record_uri)
        self.assertEqual(
            record(request).status_code,
            200,
        )


class SmsTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.sms_uri = '/test_app/views/sms/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_sms_no_message(self):
        request = self.factory.post(self.sms_uri)
        self.assertRaises(TypeError, sms, request)

    def test_sms_with_message(self):
        request = self.factory.post(self.sms_uri)
        self.assertEqual(
            sms(request, message='test').status_code,
            200,
        )

class MessageTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.message_uri = '/test_app/views/message/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_message_no_message(self):
        request = self.factory.post(self.message_uri)
        self.assertRaises(TypeError, message, request)

    def test_message_with_media(self):
        request = self.factory.post(self.message_uri)
        self.assertEqual(
            message(
                request,
                message='test',
                media='http://i.imgur.com/Qa8GVPU.gif'
            ).status_code,
            200,
        )


class DialTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.dial_uri = '/test_app/views/dial/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_dial_no_number(self):
        request = self.factory.post(self.dial_uri)
        self.assertRaises(TypeError, dial, request)

    def test_dial_with_number(self):
        request = self.factory.post(self.dial_uri)
        self.assertEqual(
            dial(request, number='+18182223333').status_code,
            200,
        )


class ConferenceTestCase(TestCase):

    def setUp(self):

        # Test URI
        self.conf_uri = '/test_app/views/conference/'

        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)

    def test_conference_no_name(self):
        request = self.factory.post(self.conf_uri)
        self.assertRaises(TypeError, conference, request)

    def test_conference_with_name(self):
        request = self.factory.post(self.conf_uri)
        self.assertEqual(
            conference(request, name='a').status_code,
            200,
        )



# -*- coding: utf-8 -*-

from types import MethodType

from django.test import TestCase
from django.contrib.auth.models import User

from django_dynamic_fixture import G

from django_twilio.models import Caller, Credential


class CallerTestCase(TestCase):
    """
    Run tests against the :class:`django_twilio.models.Caller` model.
    """

    def setUp(self):
        self.caller = G(
            Caller,
            phone_number='+15005550000',
            blacklisted=False,
        )

    def test_has_str_method(self):
        self.assertIsInstance(self.caller.__str__, MethodType)

    def test_str_returns_a_string(self):
        self.assertIsInstance(self.caller.__str__(), str)

    def test_str_doesnt_contain_blacklisted(self):
        self.assertNotIn('blacklisted', self.caller.__str__())

    def test_unicode_contains_blacklisted(self):
        self.caller.blacklisted = True
        self.caller.save()
        self.assertIn('blacklisted', self.caller.__str__())


class CredentialTests(TestCase):

    def setUp(self):
        self.user = G(User, username='test', password='pass')
        self.credentials = G(
            Credential,
            name='Test Credentials',
            account_sid='XXX',
            auth_token='YYY',
            user=self.user,
        )

    def test_str(self):
        """
        Assert that str renders how we'd like it too
        """
        self.assertEqual(
            self.credentials.__str__(),
            'Test Credentials - XXX',
        )

    def test_credentials_fields(self):
        """
        Assert the fields are working correctly
        """
        self.assertEqual(self.credentials.name, 'Test Credentials')
        self.assertEqual(self.credentials.account_sid, 'XXX')
        self.assertEqual(self.credentials.auth_token, 'YYY')
        self.assertEqual(self.credentials.user, self.user)



# -*- coding: utf-8 -*-

from django.urls import path

from . import views

# Test URLs for our ``django_twilio.decorators`` module.
urlpatterns = [
    path('decorators/response_view/', views.response_view),
    path('decorators/response_class_view/', views.ResponseView.as_view()),
    path('decorators/str_view/', views.str_view),
    path('decorators/str_class_view/', views.StrView.as_view()),
    path('decorators/bytes_view/', views.bytes_view),
    path('decorators/bytes_class_view/', views.BytesView.as_view()),
    path('decorators/verb_view/', views.verb_view),
    path('decorators/verb_class_view/', views.VerbView.as_view())
]


# -*- coding: utf-8 -*-

from urllib.parse import urljoin

from twilio.request_validator import RequestValidator

from django.test import RequestFactory


class TwilioRequestFactory(RequestFactory):

    def __init__(self, token, **defaults):
        super(TwilioRequestFactory, self).__init__(**defaults)
        self.base_url = 'http://testserver/'
        self.twilio_auth_token = token

    def _compute_signature(self, path, params):
        return RequestValidator(
            self.twilio_auth_token
        ).compute_signature(urljoin(self.base_url, path), params=params)

    def get(self, path, data={}, **extra):
        if 'HTTP_X_TWILIO_SIGNATURE' not in extra:
            extra.update({'HTTP_X_TWILIO_SIGNATURE': self._compute_signature(path, params=data)})
        return super(TwilioRequestFactory, self).get(path, data, **extra)

    def post(self, path, data={}, content_type=None, **extra):
        if 'HTTP_X_TWILIO_SIGNATURE' not in extra:
            extra.update({'HTTP_X_TWILIO_SIGNATURE': self._compute_signature(path, params=data)})
        if content_type is None:
            return super(TwilioRequestFactory, self).post(path, data, **extra)
        else:
            return super(TwilioRequestFactory, self).post(path, data, content_type, **extra)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import
from os.path import abspath, dirname, join, normpath

from setuptools import find_packages, setup
import sys


INSTALL_PYTHON_REQUIRES = []
# We are intending to keep up to date with the supported Django versions.
# For the official support, please visit:
# https://docs.djangoproject.com/en/3.0/faq/install/#what-python-version-can-i-use-with-django and you may change the version in the URL to suit your needs, and we will try to update that here too as we upgrade with django.
if sys.version_info[1] == 5:
    # py3.5 can run 1.11 < 2.2
    django_python_version_install = 'Django>=2.2,<3.0',
    INSTALL_PYTHON_REQUIRES.append(django_python_version_install)
elif sys.version_info[1] == 6:
    # py3.6 can run 1.11 < 3.1 (likely will be <4.0)
    django_python_version_install = 'Django>=2.2,<3.2',
    INSTALL_PYTHON_REQUIRES.append(django_python_version_install)
elif sys.version_info[1] == 7:
    # py3.7 is 1.11.17 < 3.1 (likely will be <4.0)
    django_python_version_install = 'Django>=2.2,<3.2'
    INSTALL_PYTHON_REQUIRES.append(django_python_version_install)
elif sys.version_info[1] == 8:
    # py3.8 is 2.2.8 < 3.1 (likely will be <4.0)
    django_python_version_install = 'Django>=2.2.8,<3.2'
    INSTALL_PYTHON_REQUIRES.append(django_python_version_install)

setup(

    # Basic package information:
    name='django-twilio',
    version='0.13.1.b2',
    packages=find_packages(),

    # Packaging options:
    zip_safe=False,
    include_package_data=True,

    # Package dependencies:
    install_requires=[
        'setuptools>=36.2',
        'twilio>=6.3.0,<7',
        'django-phonenumber-field>=0.6',
        'phonenumbers>=8.10.22',
    ] + INSTALL_PYTHON_REQUIRES,

    # Metadata for PyPI:
    author='Randall Degges',
    author_email='rdegges@gmail.com',
    maintainer="Jason Held",
    maintainer_email="jasonsheld@gmail.com",
    license='UNLICENSE',
    url='https://github.com/rdegges/django-twilio',
    keywords='twilio telephony call phone voip sms django django-twilio',
    description='Build Twilio functionality into your Django apps.',
    long_description=open(
        normpath(join(dirname(abspath(__file__)), 'README.rst'))
    ).read(),
    project_urls={
        "Documentation": "https://django-twilio.readthedocs.io/en/latest/",
        "Code": "https://github.com/rdegges/django-twilio",
        "Tracker": "https://github.com/rdegges/django-twilio/issues",
    },
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: Public Domain',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Internet :: WWW/HTTP',
    ]

)



# -*- coding: utf-8 -*-

from django.urls import include, path

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = [
    path('admin/', admin.site.urls),
    path('test_app/', include('test_project.test_app.urls')),
]




# -*- coding: utf-8 -*-
from django.test import TestCase
from django.conf import settings

from .utils import TwilioRequestFactory

from django_twilio.request import decompose, TwilioRequest
from django_twilio.exceptions import NotDjangoRequestException


class TestRequestBase(TestCase):

    def setUp(self):
        self.factory = TwilioRequestFactory(token=settings.TWILIO_AUTH_TOKEN)
        self.call_dict = {
            'CallSid': 'CSXXXX',
            'AccountSid': 'ACXXXX',
            'From': '+44123456789',
            'To': '+1123456789',
            'CallStatus': 'queued',
            'ApiVersion': '1',
            'Direction': 'outbound-api',
            'ForwardedFrom': 'False',
            'CallerName': 'Paul'
        }

        self.message_dict = {
            'MessageSid': 'MSXXXX',
            'SmsSid': 'SSXXXX',
            'AccountSid': 'ACXXXX',
            'From': '+1123456789',
            'To': '+44123456789',
            'Body': 'That\'s is no moon!',
            'NumMedia': '0'
        }


class TestDecompose(TestRequestBase):

    def test_voice_decompose_function(self):
        request = self.factory.post(
            '/test_app/decorators/verb_view/',
            self.call_dict
        )
        response = decompose(request)
        self.assertEqual(response.type, 'voice')
        self.assertEqual(response.accountsid, 'ACXXXX')
        self.assertEqual(response.from_, '+44123456789')

    def test_sms_decompose_function(self):
        request = self.factory.post(
            '/test_app/decorators/verb_view',
            self.message_dict
        )
        response = decompose(request)
        self.assertEqual(response.type, 'message')

    def test_blank_decompose_function(self):
        request = self.factory.post(
            '/test_app/decorators/verb_view',
        )
        response = decompose(request)
        self.assertEqual(response.type, 'unknown')

    def test_blank_get_decompose_function(self):
        request = self.factory.get(
            '/test_app/decorators/verb_view?messageSid=ACXXXX',
        )
        response = decompose(request)
        self.assertEqual(response.type, 'message')

    def test_raises_not_django_request_exception(self):
        request = {}
        self.assertRaises(NotDjangoRequestException, decompose, request)



# -*- coding: utf-8 -*-

from twilio.twiml.voice_response import VoiceResponse, Dial
from twilio.twiml.messaging_response import MessagingResponse

from .decorators import twilio_view


@twilio_view
def say(request, text, voice=None, language=None, loop=None):
    """
See: http://www.twilio.com/docs/api/twiml/say.

Usage::

    # urls.py
    urlpatterns = patterns('',
        # ...
        url(r'^say/$', 'django_twilio.views.say', {'text': 'hello, world!'})
        # ...
    )
    """
    r = VoiceResponse()
    r.say(text, voice=voice, language=language, loop=loop)
    return r


@twilio_view
def play(request, url, loop=None):
    """
    See: http://www.twilio.com/docs/api/twiml/play.

    Usage::

        # urls.py
        urlpatterns = patterns('',
            # ...
            url(r'^play/$', 'django_twilio.views.play', {
                    'url': 'http://blah.com/blah.wav',
            }),
            # ...
        )
    """
    r = VoiceResponse()
    r.play(url, loop=loop)
    return r


@twilio_view
def gather(request, action=None, method='POST', num_digits=None, timeout=None,
           finish_on_key=None):
    """
    See: http://www.twilio.com/docs/api/twiml/gather.

    Usage::

        # urls.py
        urlpatterns = patterns('',
            # ...
            url(r'^gather/$', 'django_twilio.views.gather'),
            # ...
        )
    """
    r = VoiceResponse()
    r.gather(action=action, method=method, numDigits=num_digits,
             timeout=timeout, finishOnKey=finish_on_key)
    return r


@twilio_view
def record(request, action=None, method='POST', timeout=None,
           finish_on_key=None, max_length=None, transcribe=None,
           transcribe_callback=None, play_beep=None):
    """
    See: http://www.twilio.com/docs/api/twiml/record.

    Usage::

        # urls.py
        urlpatterns = patterns('',
            # ...
            url(r'^record/$', 'django_twilio.views.record'),
            # ...
        )
    """
    r = VoiceResponse()
    r.record(action=action, method=method, timeout=timeout,
             finishOnKey=finish_on_key, maxLength=max_length,
             transcribe=transcribe, transcribeCallback=transcribe_callback,
             playBeep=play_beep)
    return r


@twilio_view
def sms(request, message, to=None, sender=None, action=None, method='POST',
        status_callback=None):
    """
    NOTE: Now deprecated, please use message() instead
    See: http://www.twilio.com/docs/api/twiml/sms

    Usage::

        # urls.py
        urlpatterns = patterns('',
            # ...
            url(r'^sms/$', 'django_twilio.views.sms', {
                'message': 'Hello, world!'
            }),
            # ...
        )
    """
    r = MessagingResponse()
    r.message(message, to=to, sender=sender, method='POST', action=action,
              statusCallback=status_callback)
    return r


@twilio_view
def message(request, message, to=None, sender=None, action=None,
            methods='POST', media=None, status_callback=None):
    """
    See: https://www.twilio.com/docs/api/twiml/sms/message

    Usage::

        # urls.py
        urlpatterns = patterns('',
            # ...
            url(r'^sms/$', 'django_twilio.views.message', {
                'message': 'Hello, world!',
                'media': 'http://fullpath.com/my_image.png'
            }),
            # ...
        )
    """

    r = MessagingResponse()
    r.message(message, to=to, sender=sender, method='POST',
              action=action, statusCallback=status_callback,
              media=media)

    return r


@twilio_view
def dial(request, number, action=None, method='POST', timeout=None,
         hangup_on_star=None, time_limit=None, caller_id=None):
    """
    See: http://www.twilio.com/docs/api/twiml/dial.
    """
    # Usage::
    #
    #     # urls.py
    #     urlpatterns = patterns('',
    #         # ...
    #         url(r'^dial/(?P<number>\w+)/$', 'django_twilio.views.dial'),
    #         # ...
    #     )

    r = VoiceResponse()
    r.dial(number=number, action=action, method=method, timeout=timeout,
           hangupOnStar=hangup_on_star, timeLimit=time_limit,
           callerId=caller_id)
    return r


@twilio_view
def conference(request, name, muted=None, beep=None,
               start_conference_on_enter=None, end_conference_on_exit=None,
               wait_url=None, wait_method='POST', max_participants=None):
    """
    See: http://www.twilio.com/docs/api/twiml/conference.
    """
    # Usage::
    #
    #     # urls.py
    #     urlpatterns = patterns('',
    #         # ...
    #         url(r'^conference/(?P<name>\w+)/$',
    #             'django_twilio.views.conference',
    #              {'max_participants': 10}
    #         ),
    #         # ...
    #     )
    #
    r = VoiceResponse()
    dial = Dial()
    dial.conference(name=name, muted=muted, beep=beep,
                    startConferenceOnEnter=start_conference_on_enter,
                    endConferenceOnExit=end_conference_on_exit,
                    waitUrl=wait_url, waitMethod=wait_method,
                    )
    r.append(dial)
    return r
